var react = require('react');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

// A type of promise-like that resolves synchronously and supports only one observer
const _Pact = /*#__PURE__*/(function() {
	function _Pact() {}
	_Pact.prototype.then = function(onFulfilled, onRejected) {
		const result = new _Pact();
		const state = this.s;
		if (state) {
			const callback = state & 1 ? onFulfilled : onRejected;
			if (callback) {
				try {
					_settle(result, 1, callback(this.v));
				} catch (e) {
					_settle(result, 2, e);
				}
				return result;
			} else {
				return this;
			}
		}
		this.o = function(_this) {
			try {
				const value = _this.v;
				if (_this.s & 1) {
					_settle(result, 1, onFulfilled ? onFulfilled(value) : value);
				} else if (onRejected) {
					_settle(result, 1, onRejected(value));
				} else {
					_settle(result, 2, value);
				}
			} catch (e) {
				_settle(result, 2, e);
			}
		};
		return result;
	};
	return _Pact;
})();

// Settles a pact synchronously
function _settle(pact, state, value) {
	if (!pact.s) {
		if (value instanceof _Pact) {
			if (value.s) {
				if (state & 1) {
					state = value.s;
				}
				value = value.v;
			} else {
				value.o = _settle.bind(null, pact, state);
				return;
			}
		}
		if (value && value.then) {
			value.then(_settle.bind(null, pact, state), _settle.bind(null, pact, 2));
			return;
		}
		pact.s = state;
		pact.v = value;
		const observer = pact.o;
		if (observer) {
			observer(pact);
		}
	}
}

function _isSettledPact(thenable) {
	return thenable instanceof _Pact && thenable.s & 1;
}

// Asynchronously iterate through an object that has a length property, passing the index as the first argument to the callback (even as the length property changes)
function _forTo(array, body, check) {
	var i = -1, pact, reject;
	function _cycle(result) {
		try {
			while (++i < array.length && (!check || !check())) {
				result = body(i);
				if (result && result.then) {
					if (_isSettledPact(result)) {
						result = result.v;
					} else {
						result.then(_cycle, reject || (reject = _settle.bind(null, pact = new _Pact(), 2)));
						return;
					}
				}
			}
			if (pact) {
				_settle(pact, 1, result);
			} else {
				pact = result;
			}
		} catch (e) {
			_settle(pact || (pact = new _Pact()), 2, e);
		}
	}
	_cycle();
	return pact;
}

const _iteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator"))) : "@@iterator";

const _asyncIteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))) : "@@asyncIterator";

var get = function get(obj, accessor) {
  return accessor.split('.').reduce(function (o, i) {
    return o[i];
  }, obj);
};
var getInitialState = function getInitialState(getters) {
  return {
    pages: getters.map(function () {
      return [];
    }),
    getNext: {
      data: getters.map(function () {
        return [];
      }),
      meta: {},
      nextPageForGetters: getters.map(function () {
        return 0;
      })
    },
    getNextForGetter: {
      nextPageForGetters: getters.map(function () {
        return 0;
      })
    }
  };
};
var useCombinedPagination = function useCombinedPagination(_ref) {
  var _state$getNextForGett;

  var getters = _ref.getters,
      sortKey = _ref.sortKey,
      sort = _ref.sort,
      _ref$sortDirection = _ref.sortDirection,
      sortDirection = _ref$sortDirection === void 0 ? 'desc' : _ref$sortDirection;

  var _useState = react.useState(getInitialState(getters)),
      state = _useState[0],
      setState = _useState[1];

  var _useState2 = react.useState(false),
      loading = _useState2[0],
      setLoading = _useState2[1];

  var _useState3 = react.useState([]),
      data = _useState3[0],
      setData = _useState3[1];

  var _useState4 = react.useState(0),
      resetCounter = _useState4[0],
      setResetCounter = _useState4[1];

  var _getSortKey = react.useCallback(function (hit) {
    return get(hit, sortKey);
  }, [sortKey]);

  var _isAfter = react.useCallback(function (a, b, _temp) {
    var _ref2 = _temp === void 0 ? {
      eq: true
    } : _temp,
        eq = _ref2.eq;

    if (sort) {
      return eq ? sort(a, b) >= 0 : sort(a, b) > 0;
    }

    if (sortDirection === 'asc') {
      return eq ? _getSortKey(a) >= _getSortKey(b) : _getSortKey(a) > _getSortKey(b);
    }

    return eq ? _getSortKey(a) <= _getSortKey(b) : _getSortKey(a) < _getSortKey(b);
  }, [_getSortKey, sort, sortDirection]);

  var _isBefore = react.useCallback(function (a, b, _temp2) {
    var _ref3 = _temp2 === void 0 ? {
      eq: true
    } : _temp2,
        eq = _ref3.eq;

    if (sort) {
      return eq ? sort(a, b) <= 0 : sort(a, b) < 0;
    }

    if (sortDirection === 'asc') {
      return eq ? _getSortKey(a) <= _getSortKey(b) : _getSortKey(a) < _getSortKey(b);
    }

    return eq ? _getSortKey(a) >= _getSortKey(b) : _getSortKey(a) > _getSortKey(b);
  }, [_getSortKey, sort, sortDirection]);

  var _sortPage = react.useCallback(function (hits) {
    return sort ? hits.sort(sort) : hits.sort(function (a, b) {
      return sortDirection === 'asc' ? get(a, sortKey) - get(b, sortKey) : get(b, sortKey) - get(a, sortKey);
    });
  }, [sort, sortDirection, sortKey]);

  var _mergeData = react.useCallback(function (data) {
    return _sortPage(data.reduce(function (acc, hitsForGetter) {
      return [].concat(acc, hitsForGetter);
    }, []));
  }, [_sortPage]);

  var _trimPage = react.useCallback(function (_ref4) {
    var page = _ref4.page,
        meta = _ref4.meta;
    var earliestLastHit = meta.earliestLastHit,
        firstHit = meta.firstHit;
    var trimmedPage = page.filter(function (hit) {
      return _isAfter(hit, firstHit) && _isBefore(hit, earliestLastHit);
    });
    return trimmedPage;
  }, [_isAfter, _isBefore]);

  var _getMeta = react.useCallback(function (_ref5) {
    var currentMeta = _ref5.currentMeta,
        results = _ref5.results;
    var lastHitForGetter = results[results.length - 1];
    var _currentMeta$earliest = currentMeta.earliestLastHit,
        earliestLastHit = _currentMeta$earliest === void 0 ? lastHitForGetter : _currentMeta$earliest,
        _currentMeta$firstHit = currentMeta.firstHit,
        firstHit = _currentMeta$firstHit === void 0 ? results[0] : _currentMeta$firstHit;
    return {
      firstHit: _isBefore(results[0], firstHit, {
        eq: false
      }) ? results[0] : firstHit,
      earliestLastHit: _isBefore(lastHitForGetter, earliestLastHit, {
        eq: false
      }) ? lastHitForGetter : earliestLastHit
    };
  }, [_isBefore]);

  var _shouldProcessPage = react.useCallback(function (_ref6) {
    var _data$getterIndex;

    var data = _ref6.data,
        page = _ref6.page,
        getterIndex = _ref6.getterIndex;
    return ((_data$getterIndex = data[getterIndex]) === null || _data$getterIndex === void 0 ? void 0 : _data$getterIndex.length) === 0 && page !== null;
  }, []);

  var _tidyData = react.useCallback(function (_ref7) {
    var data = _ref7.data,
        trimmedPage = _ref7.trimmedPage;
    return data.reduce(function (acc, getterData) {
      return [].concat(acc, [getterData.filter(function (hit) {
        return trimmedPage.indexOf(hit) === -1;
      })]);
    }, []);
  }, []);

  var _getData = react.useCallback(function (_ref8) {
    var getterIndex = _ref8.getterIndex,
        page = _ref8.page,
        userOptions = _ref8.userOptions,
        inputState = _ref8.inputState;

    try {
      var _temp5 = function _temp5() {
        return {
          results: results,
          newState: newState
        };
      };

      var getter = getters[getterIndex];
      var cachedPage = state.pages[getterIndex][page || 0];
      var results;

      var newState = _extends({}, state, inputState || {});

      var _temp6 = function () {
        if (cachedPage) {
          results = cachedPage;
        } else {
          return Promise.resolve(getter(page || 0, userOptions)).then(function (_getter) {
            results = _getter;
            newState.pages[getterIndex].push(results);
          });
        }
      }();

      return Promise.resolve(_temp6 && _temp6.then ? _temp6.then(_temp5) : _temp5(_temp6));
    } catch (e) {
      return Promise.reject(e);
    }
  }, [getters, state]);

  var getNext = react.useCallback(function (userOptions) {
    try {
      var _temp10 = function _temp10() {
        var page = _mergeData(newState.getNext.data);

        var trimmedPage = page.length > 0 ? _trimPage({
          page: page,
          meta: newState.getNext.meta
        }) : page;
        newState.getNext.data = _tidyData({
          data: newState.getNext.data,
          trimmedPage: trimmedPage
        });
        setState(newState);
        setLoading(false);
        setData([].concat(data, trimmedPage));
        return trimmedPage;
      };

      if (loading) {
        return Promise.resolve([]);
      }

      setLoading(true);

      var newState = _extends({}, state);

      newState.getNext.meta.firstHit = undefined;
      newState.getNext.meta.earliestLastHit = undefined;

      var _temp11 = _forTo(getters, function (getterIndex) {
        var page = newState.getNext.nextPageForGetters[getterIndex];

        var _temp7 = function () {
          var _newState$getNext, _newState$getNext$dat;

          if (_shouldProcessPage({
            data: newState.getNext.data,
            page: page,
            getterIndex: getterIndex
          })) {
            return Promise.resolve(_getData({
              getterIndex: getterIndex,
              page: page,
              userOptions: userOptions,
              inputState: newState
            })).then(function (_ref9) {
              var results = _ref9.results,
                  newStateToSave = _ref9.newState;
              newState = _extends({}, newStateToSave);

              if (results.length > 0) {
                newState.getNext.data[getterIndex] = results;
                newState.getNext.nextPageForGetters[getterIndex] = (page || 0) + 1;
                newState.getNext.meta = _getMeta({
                  currentMeta: newState.getNext.meta,
                  getterIndex: getterIndex,
                  results: results
                });
              } else {
                newState.getNext.nextPageForGetters[getterIndex] = null;
              }
            });
          } else if (((_newState$getNext = newState.getNext) === null || _newState$getNext === void 0 ? void 0 : (_newState$getNext$dat = _newState$getNext.data[getterIndex]) === null || _newState$getNext$dat === void 0 ? void 0 : _newState$getNext$dat.length) > 0) {
            newState.getNext.meta = _getMeta({
              currentMeta: newState.getNext.meta,
              getterIndex: getterIndex,
              results: newState.getNext.data[getterIndex]
            });
          }
        }();

        if (_temp7 && _temp7.then) return _temp7.then(function () {});
      });

      return Promise.resolve(_temp11 && _temp11.then ? _temp11.then(_temp10) : _temp10(_temp11));
    } catch (e) {
      return Promise.reject(e);
    }
  }, [_getData, _getMeta, _mergeData, _shouldProcessPage, _tidyData, _trimPage, data, getters, state]);
  var getNextForGetter = react.useCallback(function (getterIndex, userOptions) {
    try {
      var page = state.getNextForGetter.nextPageForGetters[getterIndex];

      if (page === null) {
        return Promise.resolve([]);
      }

      return Promise.resolve(_getData({
        getterIndex: getterIndex,
        page: page,
        userOptions: userOptions
      })).then(function (_ref10) {
        var results = _ref10.results,
            newStateToSave = _ref10.newState;

        var newState = _extends({}, newStateToSave);

        if (results.length === 0) {
          newState.getNextForGetter.nextPageForGetters[getterIndex] = null;
        } else {
          newState.getNextForGetter.nextPageForGetters[getterIndex] = page + 1;
        }

        setState(newState);
        return results;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  }, [_getData, state === null || state === void 0 ? void 0 : (_state$getNextForGett = state.getNextForGetter) === null || _state$getNextForGett === void 0 ? void 0 : _state$getNextForGett.nextPageForGetters]);
  var hasNext = react.useMemo(function () {
    var _state$getNext;

    return ((_state$getNext = state.getNext) === null || _state$getNext === void 0 ? void 0 : _state$getNext.nextPageForGetters.filter(function (i) {
      return i !== null;
    }).length) > 0;
  }, [state]);
  var reset = react.useCallback(function () {
    setState(getInitialState(getters));
    setData([]);
  }, [getters]);
  var refetch = react.useCallback(function () {
    reset();
    setResetCounter(resetCounter + 1);
  }, [reset, resetCounter]);
  react.useEffect(function () {
    if (resetCounter > 0) {
      getNext();
    }
  }, [resetCounter]);
  return {
    getNext: getNext,
    getNextForGetter: getNextForGetter,
    state: state,
    setState: setState,
    loading: loading,
    data: data,
    hasNext: hasNext,
    reset: reset,
    refetch: refetch
  };
};

exports.get = get;
exports.getInitialState = getInitialState;
exports.useCombinedPagination = useCombinedPagination;
//# sourceMappingURL=index.js.map
